# ===============================================================
# TestMatrixProperties.py: Compute matrix properties ...
# 
#                                                   November 2022 
# ===============================================================

import numpy as np # Make numpy available using np.
from numpy.linalg import matrix_rank

# ===============================================================
# Function to print two-dimensional matrices ...
# ===============================================================

def PrintMatrix(name, a):
    print("Matrix: {:s} ".format(name) );
    for row in a:
        for col in row:
            print("{:8.2f}".format(col), end=" ")
        print("")

# ===============================================================
# main method ...
# ===============================================================

def main():
    print("--- Enter TestMatrixProperties.main()        ... ");
    print("--- ======================================== ... ");

    print("--- ");
    print("--- Part 1: Create (3x3) and (3x1) test matrices ... ");
    print("--- ");

    A = np.array([ [2, 0,  0], [3,-1,  6], [4, 6, -2] ]);
    PrintMatrix("A", A);

    print("--- ");
    print("--- Matrix transpose ... ");
    print("--- ");

    B = A.T
    PrintMatrix("A^T", B );

    print("--- ");
    print("--- Matrix rank and determinant ... ");
    print("--- ");

    rank = matrix_rank(A)
    det  = np.linalg.det(A)

    print("--- Matrix A: rank = {:f}, det = {:f}  ...".format(rank, det) );

    print("--- ");
    print("--- Create Augmented Matrix (join on horizontal axis) ... ");
    print("--- ");

    C = np.concatenate((A, B), axis=0)
    PrintMatrix("C", C );

    print("--- ");
    print("--- Create Augmented Matrix (join on vertical axis) ... ");
    print("--- ");

    C = np.concatenate((A, B), axis=1)
    PrintMatrix("C", C );

    print("--- ");
    print("--- Example 2: Create (4x4) test matrix ... ");
    print("--- ");

    A = np.array([ [  2, -1, 3,  0],
                   [  4, -2, 7,  0],
                   [ -3, -4, 1,  5],
                   [  6, -6, 8,  0 ] ]);
    PrintMatrix("A", A);

    rank = matrix_rank(A)
    det  = np.linalg.det(A)

    print("--- Matrix A: rank = {:f}, det = {:f}  ...".format(rank, det) );

    print("--- ");
    print("--- Example 3: Create augmented test matrix ... ");
    print("--- ");

    a = 4;
    print("--- Parameter a = {:f} ...".format(a) );

    A = np.array( [ [ 1,  2,         -3],
                    [ 2, -1,          5],
                    [ 3,  1, (a*a - 14)] ]);

    B = np.array( [ [4], [2], [(a+2)] ] );

    PrintMatrix("A", A);
    PrintMatrix("B", B);

    print("--- ");
    print("--- Augmented test matrix ... ");
    print("--- ");

    C = np.concatenate((A, B), axis=1)
    PrintMatrix("C = [A | B]", C);

    print("--- Compute matrix rank ... ");

    rank = matrix_rank(C)
    if rank < 3:
       print("--- Matrix C (rank deficient): rank = {:f} ...".format(rank) );
    else:
       print("--- Matrix C : rank = {:f} ...".format(rank) );

    print("--- ======================================== ... ");
    print("--- Leave TestMatrixProperties.main()        ... ");

# call the main method ...

main()
